*=$1000   ; start address (use SYS4096 to start)

;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                      INITIALIZE (main program: interrupts, charset, etc.)
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

INIT            JSR LIGHTGUN_INIT       ;show the crosshair of the lightgun

SOUND_INIT      LDA #SOUNDFX_MUSIC      ;0=first tune, 1=second tune, 2=third tune, etc
                JSR $5E40               ;initialise SID (this address depends on the SID being played)


RASTERINT_INIT  SEI                     ;prevent an non maskable interrupt     
                LDA #%01111111          ;so that we can setup a raster interrupt for all our timing specific routines
                STA $DC0D               ;"Switch off" all interrupts signals from CIA-1
                AND $D011               ;
                STA $D011               ;Clear most significant bit in VIC's raster register
                LDA #250                ;Set the raster line number where interrupt should occur 
                STA $D012               ;this value will be changed in the interrupt handler, so the
                LDA #<IRQ               ;initial value does not really matter
                STA $0314               ;
                LDA #>IRQ               ;
                STA $0315               ;Set the interrupt vector to point to interrupt service routine below
                LDA #%00000001          ;
                STA $D01A               ;Enable raster interrupt signals from VIC
                CLI                     ;enable non maskable interrupt        

;CHARSET_INIT ;this label name causes problem on cbm program studio 3.8.0, 3.7.0 and a few versione before that perhaps but that isn't known for sure
CHARS_INIT      LDA $D018               ;get current charset location settings
                AND #%11110001          ;mask out unrelated bits
                ORA #%00001110          ;set charset related bits to the preferred state
                STA $D018               ;set charset to new location ($3800-$3FFF)
              
                LDA $D016               ;get current VIC values
                ORA #%00010000          ;set charset multicolor related bits to the preferred state
                STA $D016               ;    

                LDA #$80                ;DISABLE case changes caused by pressing CBM+shift
                STA $0291               ;

                        
SPRITE_INIT     LDA #$80                ;sprite trigger
                STA $07F8               ;set pointer for sprite-0 ($80 => $2000)
                ;LDA #$83 ;debug !!
                LDA #$81                ;sprite_mole
                STA SPRITE_PNTR_1       ;set pointer for sprite-1
                STA SPRITE_PNTR_2       ;set pointer for sprite-2
                STA SPRITE_PNTR_3       ;set pointer for sprite-3
                STA SPRITE_PNTR_4       ;set pointer for sprite-4
                STA SPRITE_PNTR_5       ;set pointer for sprite-5

                LDA #$83                ;sprite that shows the bounding box
                STA $07FF               ;set pointer for sprite-7 ($80 => $2000)

                LDA #15                 ;
                STA $D025               ;sprite color-1 (bitpair '01' = color defined at $d025)
                LDA #10                 ;
                STA $D026               ;sprite color-2 (bitpair '10' = color defined at $d026)

                LDA #$02                ;
                STA $D027               ;sprite 0 color (aka color-3 or bitpair '11') the sprite's only individual color

                LDA #0                  ;0=black, 15=light grey
                STA $D028               ;sprite 1 color (aka color-3 or bitpair '11') the sprite's only individual color

                LDA #1                  ;
                STA $D029               ;sprite 2 color

                STA $D02A               ;sprite 3 color
                STA $D02B               ;sprite 4 color
                STA $D02C               ;sprite 5 color
                STA $D02D               ;sprite 6 color

                LDA #$01                ;white
                STA $D02E               ;sprite 7 color

                LDA #%00000000          ;
                STA $D010               ;sprite 0-7 9th bit of X-coordinate
                LDA #%00000001          ;sprite 0 = crosshair (this must remain visible)
                STA $D015               ;sprite 0-7 enable/disable register (1=enable)
                LDA #%00000000          ;
                STA $D017               ;sprite 0-7 expand in Y direction register (1=enable)
                STA $D01D               ;sprite 0-7 expand in X direction register (1=enable)

                LDA #%00000110          ;1=background before sprite (some sprites need to be behind the background, so the moles can hide)
                STA $D01B               ;sprite/background priority

                LDA #%00000100          ;
                STA $D01C               ;sprite 0-7 MCM enable/disable register (1=MultiColorMode)


                    
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                             MAIN
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

MAIN           
GAME_CREDITS    LDA #SOUNDFX_MUSIC      ;0=first tune, 1=second tune, 2=third tune, etc
                JSR SOUNDFX_INIT        ;initialise SID (this address depends on the SID being played)

SCROLL_INIT     LDA #<MSG_SCROLL        ;initialize the scroll variables
                STA SCR_TXT_PNTR        ;
                LDA #>MSG_SCROLL        ;
                STA SCR_TXT_PNTR+1      ;

                LDA #$00                ;disable multiplexed sprites
                STA SPR_MULTIPLEXER     ;this flag is used to enable/disable the sprite multiplexer
                LDA #$81                ;                
                STA $07F9               ;set pointer for sprite-1
                LDA #$82                ;
                STA $07FA               ;set pointer for sprite-2
                LDA #%00000111          ;(sprite 0=crosshair, this one must remain visible)
                STA $D015               ;sprite 0-7 enable/disable register (1=enable)
                LDA #203                ;
                STA $D002               ;sprite-1 X-pos
                STA $D004               ;sprite-2 X-pos
                LDA #92                 ;
                STA $D003               ;sprite-1 Y-pos
                STA $D005               ;sprite-2 Y-pos


MAIN_SCREEN     LDA #0                  ;15=light grey
                STA BACKGROUND1         ;COLOR(01)
                LDA #8 ;9               ;8=orange (is more bright AND looks better then brown)   ;9=brown
                STA BACKGROUND2         ;COLOR(10)
                ;LDA #2                 ;
                ;STA BACKGROUND3        ;COLOR(11)

                LDA #7                  ;0=black, 1=white, 5=green, 9=brown, etc.
                STA COL_PRINT           ;set printing color
                LDX #0                  ;build the screen
                LDY #0                  ;
                JSR SET_CURSOR          ;
                LDA MODE_WINTER         ;determine the type of screen
                BEQ MAIN_WINTER         ;
MAIN_SUMMER     LDA #5                  ;set background and border colour to light-green (summer (grass))
                STA BORDER              ;poke x in 53280,x (bordercolour)
                STA BACKGROUND          ;poke x in 53281,x (background-0)
                LDA #<SCREEN_MAIN_SUMMER;set pointer to the text that defines the main-screen
                LDY #>SCREEN_MAIN_SUMMER;
                JSR PRINT_STRING        ;the print routine is called, so the pointed text is now printed to screen     
                JMP WAIT_FOR_REL        ;


MAIN_WINTER     LDA #1                  ;set background and border colour to white (winter (snow))
                STA BORDER              ;poke x in 53280,x (bordercolour)
                STA BACKGROUND          ;poke x in 53281,x (background-0)
                LDA #<SCREEN_MAIN_WINTER;set pointer to the text that defines the main-screen
                LDY #>SCREEN_MAIN_WINTER;
                JSR PRINT_STRING        ;the print routine is called, so the pointed text is now printed to screen     




       
WAIT_FOR_REL    JSR SCAN_KEYRELEASE     ;wait until the keyboard is released (no key pressed)
WAIT_FOR_USER   JSR SCROLL_MESSAGE      ;update the scrolling message at the bottom of the screen
                JSR GET_LIGHTGUN_POS    ;update position of crosshair on the screen
                LDA #$FF                ;we fill the debounce register with $FF, this way it will respond immediately when the trigger is pressed (no debounce delay)
                STA DEBOUNCE            ;this is required because of the scroll routine the trigger loop can't be called as much as desired
                JSR CHECK_TRIGGER       ;
                BCC WFU_01              ;
                JMP GAME_INIT           ;trigger pulled, then start game

WFU_01          JSR SCAN_KEYPRESS       ;
                CMP #KEY_F1             ;
                BNE WFU_02              ;
                JSR CALLIBRATE_LIGHTGUN ;
                JMP SCROLL_INIT         ;after callibration return to the MAIN screen

WFU_02          CMP #KEY_F3             ;toggle winter/summer mode
                BNE WFU_03              ;
                INC MODE_WINTER         ;
                LDA MODE_WINTER         ;
                AND #%00000001          ;only use lowest bit
                STA MODE_WINTER         ;
                JMP MAIN_SCREEN         ;

WFU_03          JMP WAIT_FOR_USER       ;


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////
;                                           GAME INIT
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

GAME_INIT       LDX #0                  ;build the screen
                LDY #0                  ;
                JSR SET_CURSOR          ;
                LDA MODE_WINTER         ;
                BEQ GAME_WINTER         ;

GAME_SUMMER     LDA #<SCREEN_GAME_SUMMER;set pointer to the text that defines the game-screen
                LDY #>SCREEN_GAME_SUMMER;
                JSR PRINT_STRING        ;the print routine is called, so the pointed text is now printed to screen    
                JMP GAME_INIT_01        ;

GAME_WINTER     LDA #<SCREEN_GAME_WINTER;set pointer to the text that defines the game-screen
                LDY #>SCREEN_GAME_WINTER;
                JSR PRINT_STRING        ;the print routine is called, so the pointed text is now printed to screen    

GAME_INIT_01    LDA #$01                ;enable multiplexed sprites
                STA SPR_MULTIPLEXER     ;this flag is used to enable/disable the sprite multiplexer
                LDA #%00000111          ;enable sprites used by the multiplexer (and crosshair)
                STA $D015               ;sprite 0-7 enable/disable register (1=enable)


                LDA #%00011111          ;all moles go up... (give the player a chance of hitting something directly from the start)
                STA MOLE_UP             ;
                
                LDA #120        ;l60                 ;the clock at the beginning of the game
                STA GAME_TIME           ;
                LDA #60                 ;the first hit should be made easy... and this allows the start tune to be fully played
                STA MOLE_DELAY          ;


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////
;                                              GAME
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

GAME_START      LDA #SOUNDFX_STARTEND   ;0=first tune, 1=second tune, 2=third tune, etc
                JSR SOUNDFX_INIT        ;initialise SID (this address depends on the SID being played)
                LDA #$00                ;reset score
                STA NEW_SCORE           ;
                STA NEW_MISSED          ;
          
GAME_LOOP       LDA #$0                 ;clear port to enable crude keyboard readout
                STA $DC00               ;
                LDA DATAPORTB           ;check keyboard inputs
                CMP #$7F                ;RUN/STOP-key pressed?
                BNE GAME_LOOP_01        ;
                JMP MAIN                ;


GAME_LOOP_01    JSR GET_LIGHTGUN_POS    ;
                ;LDX #0                  ;debug only (show the user where he/she is pointing)
                ;LDY #0                  ;debug only
                ;JSR SET_CURSOR          ;debug only
                ;LDA LIGHTGUN_X          ;debug only
                ;JSR PRINT_DEC           ;debug only
                ;LDX #0                  ;debug only
                ;LDY #1                  ;debug only
                ;JSR SET_CURSOR          ;debug only
                ;LDA LIGHTGUN_Y          ;debug only
                ;JSR PRINT_DEC           ;debug only

                JSR CHECK_TRIGGER       ;this routine will return with C=0 when nothing is pressed, C=1 when trigger is pressed (routine debounces and has edge detection)
                BCC UPDATE_TIME         ;when the trigger isn't pulled we do not need to check for hits, so we have plenty of time to update the time and score
                JSR CHECK_HIT           ;check if the user is hitting something

UPDATE_TIME     LDX #36                 ;
                LDY #2                  ;
                JSR SET_CURSOR          ;
                LDA GAME_TIME           ;
                JSR PRINT_DEC           ;write the value to screen for diagnostics purposes

                LDA GAME_TIME           ;check if the game_time has passed
                BNE UPDATE_SCORE        ;and if so, then exit the game
                JMP GAME_OVER           ;we have only one level, so when complete it is game over.
                           
UPDATE_SCORE    LDX #36                 ;
                LDY #6                  ;
                JSR SET_CURSOR          ;
                LDA NEW_SCORE           ;
                JSR PRINT_DEC           ;write the value to screen for diagnostics purposes

UPDATE_MISSED   LDX #36                 ;
                LDY #10                 ;
                JSR SET_CURSOR          ;
                LDA NEW_MISSED          ;
                JSR PRINT_DEC           ;write the value to screen for diagnostics purposes

                LDA NEW_MISSED          ;
                CMP #MISS_LIMIT         ;when you reach (or exceed) the miss limit, the game stops
                BCC UPDATE_MOLES        ;
                JMP GAME_OVER           ;



UPDATE_MOLES    LDA MOVE_MOLE           ;check if mole movement is allowed (this is nothing more then a method of delaying the animation)
                BNE UPDATE_M_01         ;
                JMP GAME_LOOP           ;
UPDATE_M_01     LDA #$00                ;clear the flag that brought us here
                STA MOVE_MOLE           ;
                LDX #1                  ;mole 1
                JSR MOLE_MOVE           ;update mole movement
                STA Y_POS_MOLE_1        ;
                LDX #2                  ;mole 2
                JSR MOLE_MOVE           ;update mole movement
                STA Y_POS_MOLE_2        ;
                LDX #3                  ;mole 3
                JSR MOLE_MOVE           ;update mole movement
                STA Y_POS_MOLE_3        ;
                LDX #4                  ;mole 4
                JSR MOLE_MOVE           ;update mole movement
                STA Y_POS_MOLE_4        ;
                LDX #5                  ;mole 5
                JSR MOLE_MOVE           ;update mole movement
                STA Y_POS_MOLE_5        ;
UPDATE_M_EXIT                           ;


GENERATE_MOLES  DEC MOLE_DELAY          ;
                BEQ GENERATE_M_01       ;
                JMP GAME_LOOP           ;
GENERATE_M_01   LDA GAME_TIME           ;the speed of which the moles move up/down is
                CLC                     ;based on the remaining time, the less time is left
                ROR                     ;the more hectic the game becomes
                CLC                     ;
                ROR                     ;
                CLC                     ;
                ROR                     ;
                ADC #10                 ;
                STA MOLE_DELAY          ;

                                        ;get "random" value to determine if the mole should be visible or not                
GENERATE_M_02   LDA $DC04               ;CIA#1  Timer A  Lo byte     
                EOR $DC05               ;CIA#1  Timer A  Hi byte 
                EOR $DD04               ;CIA#2  Timer A  Lo byte 
                ADC $DD05               ;CIA#2  Timer A  Hi byte 
                EOR $DD06               ;CIA#2  Timer B  Lo byte 
                EOR $DD07               ;CIA#2  Timer B  Hi byte 
                AND #%00011111          ;mask out the bits that are not required
                STA MOLE_UP             ;this reg now contains the new status of the moles (up/down)

GENERATE_M_04   BEQ GENERATE_M_05       ;when there is at least 1 mole up, then
                LDA #SOUNDFX_MOLECHANGE ;make the sound of anoying moles
                JSR SOUNDFX_INIT        ;initialise SID (this address depends on the SID being played)

GENERATE_M_05   LDA #$00                ;
                STA MOLE_HIT            ;
GENERATE_M_EXIT JMP GAME_LOOP           ;
                

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////
;                                            GAME OVER
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\


GAME_OVER       LDA #$00                ;disable multiplexed sprites
                STA SPR_MULTIPLEXER     ;this flag is used to enable/disable the sprite multiplexer
                LDA #%00000001          ;disable sprites used by the multiplexer (crosshair remains visible)
                STA $D015               ;sprite 0-7 enable/disable register (1=enable)


                LDA #SOUNDFX_STARTEND   ;0=first tune, 1=second tune, 2=third tune, etc
                JSR SOUNDFX_INIT        ;initialise SID (this address depends on the SID being played)

                JSR CLEARSCREEN         ;clear the screen, because we are not gonna write a whole screen

                LDX #0                  ;build the screen
                LDY #0                  ;
                JSR SET_CURSOR          ;
                LDA #<SCREEN_GAMEOVER   ;set pointer to the text that defines the main-screen
                LDY #>SCREEN_GAMEOVER   ;
                JSR PRINT_STRING        ;the print routine is called, so the pointed text is now printed to screen   

                LDX #0                  ;build the screen
                LDY #12                 ;
                JSR SET_CURSOR          ;
                LDA #<TXT_SCORE         ;
                LDY #>TXT_SCORE         ;
                JSR PRINT_STRING        ;the print routine is called, so the pointed text is now printed to screen   

                LDX #25                 ;
                LDY #12                 ;
                JSR SET_CURSOR          ;
                LDA NEW_SCORE           ;prepare showing the score
                JSR PRINT_DEC           ;the print routine is called

                LDX #22                 ;
                LDY #14                 ;
                JSR SET_CURSOR          ;
                LDA NEW_MISSED          ;prepare showing the score
                JSR PRINT_DEC           ;the print routine is called

                LDA #3                  ;
                STA GAME_TIME           ;
GAME_OVER_00    LDA GAME_TIME           ;check if the game-over-screen-time has passed
                BNE GAME_OVER_00        ;and if so, then this game-over-screen

                ;do not leave this screen yet, the user must be able to show his high score to others
GAME_OVER_01    JSR GET_LIGHTGUN_POS    ;update position of crosshair on the screen
                JSR CHECK_TRIGGER       ;this routine will return with C=0 when nothing is pressed, C=1 when trigger is pressed
                BCC GAME_OVER_01        ;when the trigger isn't pulled keep looping

                JMP MAIN                ;


TXT_SCORE    ;0123456789012345678901234567890123456789
        TEXT '     your managed to hit --- moles      '
        TEXT '                                        '
        TEXT '           you missed ---               '
        TEXT '                                        '
        TEXT '       better luck next time...         '
        BYTE 0


;...............................................................................
;the code below was never finished, and should be cleaned up, it is left here for future reference
;because this code is never called, it means that the user cannot enter his/hers highscore
;which isn't really important because the high scors aren't saved anyway.


;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                                     SUBROUTINES
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

;-------------------------------------------------------------------------------
;this routine will calculate the new position of the mole depeding on the status
;of the mole positioning bit as held in  "MOLE_UP"
;
; LDX #1                ;mole 1
; JSR MOLE_MOVE         ;update mole movement, new position is stored in A
; STA Y_POS_MOLE_1      ;save a to sprite position register
;...............................................................................

MOLE_TRAVEL = 15 ;the distance that a mole moves up/down

MOLE_MOVE       TXA                     ;save the X reg
                TAY                     ;so we can calculate the mask
                LDA #%00000000          ;
                SEC                     ;set carry
MOLE_MASK       ROL                     ;
                DEY                     ;
                BNE MOLE_MASK           ;
                AND MOLE_UP             ;
                BNE MOLE_MOVE_UP        ;

MOLE_MOVE_DOWN  LDA #MOLE_TRAVEL-1      ;check if the mole is moving from the end position towards the start position,
                CMP MOLE_POS,X          ;if this is the case then the mole is moving down because it wasn't hit. Therefore
                BNE MOLE_MOVE_D1        ;we should increment the miss counter.
                INC NEW_MISSED          ;

                TXA                     ;
                PHA                     ;
                TYA                     ;
                PHA                     ;
                LDX #36                 ;update screen missed counter
                LDY #10                 ;
                JSR SET_CURSOR          ;
                LDA NEW_MISSED          ;
                JSR PRINT_DEC           ;write the value to screen
                PLA                     ;
                TAY                     ;
                PLA                     ;
                TAX                     ;


MOLE_MOVE_D1    LDA #MOLE_TRAVEL        ;check end position
                CMP MOLE_POS,X          ;we use the carry to do the math
                LDA #0                  ;
                ADC MOLE_POS,X          ;
                JMP MOVE_EXIT           ;
MOLE_MOVE_UP    LDA #0                  ;check start position
                CMP MOLE_POS,X          ;we use the carry to do the math
                LDA MOLE_POS,X          ;
                SBC #0                  ;
MOVE_EXIT       STA MOLE_POS,X          ;
                CLC                     ;
                ADC DEF_POSY,X          ;
                RTS

;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                         GAME variables
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

NEW_SCORE       BYTE $00
NEW_MISSED      BYTE $00

MODE_WINTER     BYTE $00 ;0=WINTER, 1=SUMMER

GAME_TIME       BYTE $00
MOLE_UP         BYTE $00 ;this indicates a mole travelling up or down (1=up, 0=down)
MOLE_HIT        BYTE $00 ;this indicates a mole being hit (1=hit)
MOLE_DELAY      BYTE $10 ;this variable is used to slow time the mole movement

; do not change the order of these variables !!
MOLE_POS        BYTE $00 ;work register
MOLE_POS_1      BYTE $00
MOLE_POS_2      BYTE $00
MOLE_POS_3      BYTE $00
MOLE_POS_4      BYTE $00
MOLE_POS_5      BYTE $00
; do not change the order of these variables !!
DEF_POSY        BYTE $00                        ;default mole Y position
DEF_POSY1       BYTE DEFAULT_YM1
DEF_POSY2       BYTE DEFAULT_YM2
DEF_POSY3       BYTE DEFAULT_YM3
DEF_POSY4       BYTE DEFAULT_YM4
DEF_POSY5       BYTE DEFAULT_YM5

; do not change the order of these variables !!
DEF_POSX_L      BYTE $00                        ;mole bounding box left side limit
DEF_POSX_L1     BYTE DEFAULT_XM1-HIT_AREA_LEFT
DEF_POSX_L2     BYTE DEFAULT_XM2-HIT_AREA_LEFT
DEF_POSX_L3     BYTE DEFAULT_XM3-HIT_AREA_LEFT
DEF_POSX_L4     BYTE DEFAULT_XM4-HIT_AREA_LEFT
DEF_POSX_L5     BYTE DEFAULT_XM5-HIT_AREA_LEFT
; do not change the order of these variables !!
DEF_POSX_R      BYTE $00                        ;mole bounding box right side limit
DEF_POSX_R1     BYTE DEFAULT_XM1+HIT_AREA_RIGHT
DEF_POSX_R2     BYTE DEFAULT_XM2+HIT_AREA_RIGHT
DEF_POSX_R3     BYTE DEFAULT_XM3+HIT_AREA_RIGHT
DEF_POSX_R4     BYTE DEFAULT_XM4+HIT_AREA_RIGHT
DEF_POSX_R5     BYTE DEFAULT_XM5+HIT_AREA_RIGHT
; do not change the order of these variables !!
DEF_POSY_T      BYTE $00                        ;mole bounding box top side limit
DEF_POSY_T1     BYTE DEFAULT_YM1-HIT_AREA_TOP
DEF_POSY_T2     BYTE DEFAULT_YM2-HIT_AREA_TOP
DEF_POSY_T3     BYTE DEFAULT_YM3-HIT_AREA_TOP
DEF_POSY_T4     BYTE DEFAULT_YM4-HIT_AREA_TOP
DEF_POSY_T5     BYTE DEFAULT_YM5-HIT_AREA_TOP
; do not change the order of these variables !!
DEF_POSY_B      BYTE $00                        ;mole bounding box bottom side limit
DEF_POSY_B1     BYTE DEFAULT_YM1+HIT_AREA_BOTTOM
DEF_POSY_B2     BYTE DEFAULT_YM2+HIT_AREA_BOTTOM
DEF_POSY_B3     BYTE DEFAULT_YM3+HIT_AREA_BOTTOM
DEF_POSY_B4     BYTE DEFAULT_YM4+HIT_AREA_BOTTOM
DEF_POSY_B5     BYTE DEFAULT_YM5+HIT_AREA_BOTTOM




;===============================================================================
;Raster based scroll routine
;calling this routine advances the scroll message 1 pixel to the left
;this routine should be called continuously
;...............................................................................
SCROLL_MESSAGE
SCRL_OPEN       LDA $D012               ;
                CMP #242                ;
                BNE SCRL_OPEN           ;
             ;   LDA BACKGROUND          ;save current background color
             ;   STA BACKGROUNDCOL       ;
             ;   LDA #$01                ;set new background color for scrolling text
             ;   STA BACKGROUND          ;
                LDA SCROLL_X            ;the actual pixel movement is defined here
                AND #%00010111          ;the scroll position is defined in the lower 3 bits
                EOR #%00010111          ;but the scroll must be to the left so we invert the counter
                STA $D016               ;

SCRL_CLOSE      LDA $D012               ;
                CMP #251                ;
                BNE SCRL_CLOSE          ;
            ;    LDA BACKGROUNDCOL       ;get original color
            ;    STA BACKGROUND          ;
                LDA $D016               ;stop scrolling area
                AND #%11010000          ;
                ORA #%00001000          ;
                STA $D016               ;
SCRL_START      LDA SCROLL_X            ;
                CLC                     ;
                ADC #%00001000          ;the upper 5 bits of this reg are used for the delay counter, so by adding this value we actually add one to our delay counter (bit of a hassle but we save one byte formthe zero page)
                STA SCROLL_X            ;
                CMP #%00001000          ;when the SCROLL_X register exceeds this value we update the scroll
                BCS SCRL_00             ;
                JMP SCRL_END            ;

SCRL_00         AND #%00000111          ;the scroll position is defined in the lower 3 bits
                STA SCROLL_X            ;reset our delay counter  
                INC SCROLL_X            ;increment the pixel scroll position
                LDA SCROLL_X            ;
                CMP #$08                ;we scroll in steps of seven pixels
                BNE SCRL_END            ;so when there are seven pixels shifted, we print a new line of characters that is one character advanced

SCRL_01         LDA #$00                ;reset our pixel counter
                STA SCROLL_X            ;
                CLC                     ;increment the pointer to our text table, so that the text advances
                LDA #$01                ;
                ADC SCR_TXT_PNTR        ;we use the ADC instruction because the INC does not generate a Carry (which is easy to forget and creates silly bugs... oops, but it's fixed now)
                STA SCR_TXT_PNTR        ;
                BCC SCRL_02             ;check for overflow, if so, we must inc the highbyte as well
                INC SCR_TXT_PNTR+1      ;

SCRL_02         LDA #<SCROLLSCREEN      ;set addres pointer to the first charposition of the scroll line
                STA SCR_ADR_PNTR        ;
                LDA #>SCROLLSCREEN      ;
                STA SCR_ADR_PNTR+1      ;
                LDY #39                 ;the number of chars on the scroll line
                LDA (SCR_TXT_PNTR),Y    ;check if we've reached the end-of-table marker
                BNE SCRL_03             ;
                JSR SCROLL_INIT         ;end of text marker detected, we must reset the text pointer to the beginning of the table
SCRL_03         DEY                     ;
                LDA (SCR_TXT_PNTR),Y    ;
                STA (SCR_ADR_PNTR),Y    ;
                TYA                     ;move Y to A so we set/clear the Z-flag
                BNE SCRL_03             ;

SCRL_END        RTS 

BACKGROUNDCOL   BYTE $00

MSG_SCROLL      TEXT '                                       '  ;this line is required to scroll in the first character nice and clean
                TEXT 'welcome to -smack-a-mole V2- for the c64 (2016). this game only works with a crt (lcd is not supported). ' 
                TEXT 'this version is easier to hit but allows a more interesting gameplay, too many failures now simply mean: game over. '
                TEXT 'this game also supports the stack light rifle and magnum light phaser compatible guns (so if you like to,you can also shoot those ugly moles) '
                TEXT '      '
                TEXT 'programming and hardware design:jan derogee,idea:richard lagendijk,music:ben hayes (from the game:ruff and reddy). '
                TEXT 'many thanks to arthur jordison for writing -cbm program studio- (the development environment used for writing this game) a very versatile tool for all cbm related code development. '
                TEXT 'cbm program studio is completely free and works for all cbm (8-bit) computer models,so if you like to program (basic or assembly) you should check this out. '
                TEXT '      '
                TEXT 'smack-a-mole is pretty straight forward: hit (lighthammer) or shoot (lightgun) as many moles as you can before the time expires. '
                TEXT 'moles that are already going down can no longer be hit. a mole that is not hit and goes down into it's hole,is also registered as a missed hit. '
                TEXT 'when the total amount of misses has been reached its game over. if you want to quit the game press run/stop. '
                TEXT 'good luck but don't hit those moles to hard or you might damage your screen. '
                TEXT '      '
                TEXT 'make sure that you calibrate your hammer/lightgun for the best accuracy '
                TEXT '                                       '  ;this line is required to scroll out the last character nice and clean
                BYTE 0



;-------------------------------------------------------------------------------
;this routine will check for a collision with a mole that is up
; JSR CHECK_HIT
;...............................................................................
CHECK_HIT       
CHECK_HIT_CRT   LDX #5                  ;check if the "hammer" position overlaps the position of a mole
                LDY #%11011111          ;the mask used for the MOLE_UP
                
CHECK_HIT_C01   TYA                     ;shift the mask
                SEC                     ;one position to the right
                ROR                     ;
                TAY                     ;

                LDA LIGHTGUN_X          ;the gun's Y-position including offset correction
                CMP DEF_POSX_L,X        ;the leftside (X-beginning) of the box
                BCC CHECK_HIT_C02       ;

                LDA LIGHTGUN_X          ;the gun's Y-position including offset correction
                CMP DEF_POSX_R,X        ;the rightside (X-end) of the box
                BCS CHECK_HIT_C02       ;

                LDA LIGHTGUN_Y          ;the gun's Y-position including offset correction
                CMP DEF_POSY_T,X        ;the topside (Y-beginning) of the box
                BCC CHECK_HIT_C02       ;exit when outside of the box

                LDA LIGHTGUN_Y          ;the gun's Y-position including offset correction
                CMP DEF_POSY_B,X        ;the bottomside (Y-end) of the box
                BCS CHECK_HIT_C02       ;exit when outside of the box

                TYA                     ;get mask
                EOR #%11111111          ;invert mask
                AND MOLE_UP             ;check if the mole is "up", if not then we may
                BEQ CHECK_HIT_C02       ;not count this as a hit, so no valid collission

                TYA                     ;mask out the current flag to "knock down" the mole
                AND MOLE_UP             ;
                STA MOLE_UP             ;
                ;INC BORDER             ;debug only
                TYA                     ;
                EOR #%11111111          ;invert mask
                STA MOLE_HIT            ;

                LDA #MOLE_TRAVEL        ;when a mole is hit, it goes down instantly
                STA MOLE_POS,X          ;
                JMP CHECK_HIT_HIT       ;
        
CHECK_HIT_C02   DEX                     ;
                BNE CHECK_HIT_C01       ;
                JMP CHECK_HIT_MISS      ;                       

                ;........................

CHECK_HIT_LCD   ; LCD screens do not work with lightguns for many reasons
                ; they also might have a very anoying lagg, due to image processing inside the LCD screen
                ; therefore the support for LCD is scrapped (simply too many pitfalls)
                ; Suporting this would require at least: screen lag compensation ? delay detection for ... frames or msec.
                ; flashing the screen (black with 1 sprite fully white (just like the Nintendo zapper works)) to overcome
                ; the absence of of a single dot scanning the screen (this produces an ugly flickering when the trigger is pulled)
                ; All in all, a lot of work... (for a screentype nobody dares to hit with a hammer)

                ;........................

CHECK_HIT_HIT   INC NEW_SCORE           ;increment score by one
                LDA #SOUNDFX_HIT        ;0=first tune, 1=second tune, 2=third tune, etc
                JMP CHECK_HIT_EXIT

CHECK_HIT_MISS  INC NEW_MISSED          ;increment missed score by one
                LDA #SOUNDFX_MISS       ;0=first tune, 1=second tune, 2=third tune, etc
CHECK_HIT_EXIT  JSR SOUNDFX_INIT        ;initialise SID (this address depends on the SID being played)
                RTS                     ;
